/* 奥村晴彦『C言語による最新アルゴリズム事典』技術評論社，1991年，ISBN4-87408-414-1，2400円
 * http://oku.edu.mie-u.ac.jp/~okumura/algo/
 */

/************ loggamma(x) -- gamma.c より再掲 *************/
double loggamma(double x);  /* ガンマ関数の対数 */;
double p_gamma(double a, double x, double loggamma_a);
double q_gamma(double a, double x, double loggamma_a);
double p_chisq(double chisq, int df);  /* カイ2乗分布の下側確率 */
double q_chisq(double chisq, int df);  /* カイ2乗分布の上側確率 */;
#define LOG_PI 1.14472988584940017  /* $\log_e \pi$ */
double erf(double x);  /* Gaussの誤差関数 ${\rm erf}(x)$ */
double erfc(double x);  /* $1 - {\rm erf}(x)$ */
double p_normal(double x);  /* 標準正規分布の下側確率 */
double q_normal(double x);  /* 標準正規分布の上側確率 */
